import {
  ModalService, Modal, Form, Select, Grid, IButtonEvent, Button, DialogService, SelectTree, Column, SelectMultiple,
} from '@zeedhi/common';
import {
  Loader, Metadata, IEventParam, Http, IDictionary, I18n, Config,
} from '@zeedhi/core';
import { PmeMask } from '@/config';

export class Pla01308UtilizacaoProdutosController {
  private utilities = Loader.getInstance('Utilities');
  
  private utilitiesPLA = Loader.getInstance('UtilitiesPLA');
  
  private appController = Loader.getInstance('AppController');

  public modalUtilizacaoProdutos!: Modal;
  
  public prato = Loader.getInstance('Pla01300PratoController');
  
  public alteracaoReceitas = Loader.getInstance('AlteracaoReceitas');
  
  public nrlistaserv = '';
  
  public listaFilter = {
    CDFILCENTPME: '',
    CDPRODUTO: '',
    FORMA: '',
    GRUPFILI: '',
    NRLISTASERV: '',
    PRATOFINAL: '',
    PRATOINICIAL: '',
    SERVICO: [] as string[],
    TIPOCARD: [] as string[],
    TPRECEITA: [] as string[],
    TPRECEITA1: [] as string[],
    UNIDADE: [] as string[],
  };
  
  private async createModalUtilizacaoProdutos() {
    this.modalUtilizacaoProdutos = await ModalService.createFromJson(
      'planejamento/prato/modals/pla01308_utilizacao_produtos', true,
    );
  }
  
  public closeModalUtilizacaoProdutos() {
    this.modalUtilizacaoProdutos.hide();
    this.modalUtilizacaoProdutos.destroy();
    this.modalUtilizacaoProdutos = this.utilities.modalEmpty;
    if (this.nrlistaserv !== '') {
      const row = {
        NRLISTASERV: this.nrlistaserv,
      };
      Http.post('/deleteLista', { requestType: 'Row', row }).then(() => {
        this.nrlistaserv = '';
        this.listaFilter = {
          CDFILCENTPME: '',
          CDPRODUTO: '',
          FORMA: '',
          GRUPFILI: '',
          NRLISTASERV: '',
          PRATOFINAL: '',
          PRATOINICIAL: '',
          SERVICO: [] as string[],
          TIPOCARD: [] as string[],
          TPRECEITA: [] as string[],
          TPRECEITA1: [] as string[],
          UNIDADE: [] as string[],
        };
      });
    }
  }
  
  public async openModalUtilizacaoProdutos() {
    if (!this.modalUtilizacaoProdutos) {
      this.createModalUtilizacaoProdutos().then(() => {
        this.modalUtilizacaoProdutos.show();
      });
    } else {
      await this.modalUtilizacaoProdutos.show();
    }
  }

  public fullscreen({ component }: IButtonEvent) {
    this.modalUtilizacaoProdutos.fullscreen = !this.modalUtilizacaoProdutos.fullscreen;
    component.iconName = (this.modalUtilizacaoProdutos.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }

  public inicContainer() {
    // const project = localStorage.getItem('PRODUCT_ID');
    
    const tpReceitaCard = Metadata.getInstance<Select>('TPRECEITA');
    const tpReceita = Metadata.getInstance<Select>('TPRECEITA1');
    const cardapio = Metadata.getInstance<Select>('TIPOCARD');
    // const cdtpcard = Metadata.getInstance<Select>('CDTIPOCARD');
    const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
    const cdtpcard = this.utilities.returnColumn(grid, 'CDTIPOCARD') as Column;

    if (this.utilitiesPLA.isMerenda()) {
      tpReceitaCard.isVisible = true;
      tpReceita.isVisible = false;
      cdtpcard.isVisible = true;
    } else {
      tpReceitaCard.isVisible = false;
      tpReceita.isVisible = true;
      cardapio.isVisible = false;
    }
  }
  
  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValue: 'CDPRODUTO',
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/find/produtoNiveisEstoque',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
  
  public setFilterService({ component }: IEventParam<Select>) {
    const form: Form = Metadata.getInstance('formFiltroUtilizacaoProd');
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const row = form.value;
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: row.UNIDADE.length > 0 ? row.UNIDADE.join('_') : 'T',
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnid.value ? agrupUnid.value : 'T',
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: this.utilitiesPLA.isMerenda() ? '' : 'N',
      },
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: localStorage.getItem('CDOPERADOR') ? localStorage.getItem('CDOPERADOR') : 'OPERADOR_LOGADO',
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public changeUnityServ({ component }: IEventParam<SelectMultiple>) {
    // const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
    const serv = Metadata.getInstance<Select>('SERVICO');
    serv.setValue('');

    this.setFilterUtilizacaoProdutos({ component });
  }
  
  public setFilterUnidade({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const agrupUnidValue = agrupUnid.value ? agrupUnid.value : 'T';
    
    const filter = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: 'OPERADOR_LOGADO',
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterUtilizacaoProdutos({ component }: IDictionary) {
    const form: Form = Metadata.getInstance('formFiltroUtilizacaoProd');
    const row = form.value;
    let tpReceitaCard = Metadata.getInstance<Select>('TPRECEITA');
    const tpReceita = Metadata.getInstance<Select>('TPRECEITA1');
    const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
    grid.selectedRows = [];
    row.NRLISTASERV = '';
    
    if (tpReceitaCard === null) tpReceitaCard = tpReceita;
    if (row.TPRECEITA === null) row.TPRECEITA = row.TPRECEITA1;

    row.CDFILCENTPME = this.appController.getCdCentral();
        
    if (((tpReceitaCard.value && tpReceitaCard.value.length > 0)) && row.PRATOINICIAL && row.PRATOFINAL) {
      const nm = component.name;
      const tprece = row.TPRECEITA;
      if (nm !== 'PRATOINICIAL' && nm !== 'PRATOFINAL' && nm !== 'CDPRODUTO') {
        // logica pra carregar o grid apenas quando os campos filtrados mudarem
        
        if ((nm === 'TPRECEITA' || nm === 'TPRECEITA1') && this.nrlistaserv !== ''
          && tprece.includes('S') && this.listaFilter.TPRECEITA?.includes('S')) {
          this.filterGridProd();
        } else {
          this.setGridColumns(row);
          // if (tprece.includes('T')
          // && (nm === 'GRUPFILI' || nm === 'UNIDADE' || nm === 'SERVICO' || nm === 'TIPOCARD')) {
          if (tprece.includes('T') && (nm === 'TPRECEITA' || nm === 'TPRECEITA1'
            || nm === 'GRUPFILI' || nm === 'TIPOCARD')) {
            this.loadListaServ();
          } else if (tprece.includes('S') && (nm === 'TPRECEITA' || nm === 'TPRECEITA1'
            || nm === 'GRUPFILI' || nm === 'UNIDADE' || nm === 'SERVICO')) {
            this.loadListaServ();
          } else if (tprece.includes('L') && (nm === 'TPRECEITA' || nm === 'TPRECEITA1'
            || nm === 'GRUPFILI' || nm === 'UNIDADE')) {
            this.loadListaServ();
          } else {
            this.filterGridProd();
          }
        }
      } else if (this.nrlistaserv === '' && (tprece.includes('T') || tprece.includes('S') || tprece.includes('L'))) {
        this.loadListaServ();
      } else {
        this.setGridColumns(row);
        this.filterGridProd();
      }
    }
    
    grid.datasource.data = [];
    grid.datasource.total = 0;
    grid.datasource.filter = [];
  }
  
  private setGridColumns(row: IDictionary) {
    const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
    const cdtpcard = this.utilities.returnColumn(grid, 'CDTIPOCARD') as Column;
    const nmservico = this.utilities.returnColumn(grid, 'NMSERVICO') as Column;
    cdtpcard.isVisible = false;
    nmservico.isVisible = false;
    if (row.TPRECEITA.includes('T')) {
      cdtpcard.isVisible = true;
    }
    
    if (row.TPRECEITA.includes('S')) {
      nmservico.isVisible = true;
    }
  }
  
  private loadListaServ() {
    const form: Form = Metadata.getInstance('formFiltroUtilizacaoProd');
    const row = form.value;
    const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
    grid.datasource.loading = true;
    
    if (this.nrlistaserv !== '') {
      const newRow = {
        NRLISTASERV: this.nrlistaserv,
      };
      Http.post('/deleteLista', { requestType: 'Row', row: newRow });
    }
    
    if (row.TPRECEITA.includes('S') || row.TPRECEITA.includes('L') || row.TPRECEITA.includes('T')) {
      Http.post('/prato/retornaListaServ', { requestType: 'Row', row }).then((response) => {
        const result = response.data.dataset.result.data;
        this.nrlistaserv = result;
        this.listaFilter = {
          CDFILCENTPME: row.CDFILCENTPME,
          CDPRODUTO: row.CDPRODUTO,
          FORMA: row.FORMA,
          GRUPFILI: row.GRUPFILI,
          NRLISTASERV: row.NRLISTASERV,
          PRATOFINAL: row.PRATOFINAL,
          PRATOINICIAL: row.PRATOINICIAL,
          SERVICO: row.SERVICO,
          TIPOCARD: row.TIPOCARD,
          TPRECEITA: row.TPRECEITA,
          TPRECEITA1: row.TPRECEITA1,
          UNIDADE: row.UNIDADE,
        };
        this.filterGridProd();
      // }).finally(() => {
        // grid.datasource.loading = false;
      });
    } else {
      this.filterGridProd();
    }
  }
  
  private filterGridProd() {
    const form: Form = Metadata.getInstance('formFiltroUtilizacaoProd');
    const row = form.value;
    const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
    grid.datasource.loading = true;
    
    const tpReceitaCard = Metadata.getInstance<Select>('TPRECEITA');
    const conditions = [
      {
        name: 'P_CDPRATOINI',
        operator: '=',
        value: row.PRATOINICIAL,
      },
      {
        name: 'P_CDPRATOFIN',
        operator: '=',
        value: row.PRATOFINAL ? `${row.PRATOFINAL}Z` : null,
      },
      {
        name: 'P_CDFILAUXPRAT',
        operator: '=',
        value: 'P',
      },
      {
        name: 'P_CDPRODUTO',
        operator: '=',
        value: row.CDPRODUTO ? row.CDPRODUTO : 'T',
      },
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'P_RECEFILI',
        operator: '=',
        value: tpReceitaCard.value.indexOf('L') === -1 ? 'N' : 'S',
      },
      {
        name: 'P_RECETPCARD',
        operator: '=',
        value: tpReceitaCard.value.indexOf('T') === -1 ? 'N' : 'S',
      },
      {
        name: 'P_RECEPADR',
        operator: '=',
        value: tpReceitaCard.value.indexOf('P') === -1 ? 'N' : 'S',
      },
      {
        name: 'P_RECESERV',
        operator: '=',
        value: tpReceitaCard.value.indexOf('S') === -1 ? 'N' : 'S',
      },
      {
        name: 'TEXTO_PADRAO',
        operator: '=',
        value: I18n.translate('pme::padrao'),
      },
      {
        name: 'TEXTO_TPCARD',
        operator: '=',
        value: I18n.translate('pme::tipoDeCardapio'),
      },
      {
        name: 'P_CDTIPOCARD',
        operator: '=',
        value: row.TIPOCARD && row.TIPOCARD.length > 0 ? row.TIPOCARD.join('_') : 'T',
      },
      {
        name: 'P_NRLISTASERV',
        operator: '=',
        value: this.nrlistaserv,
      },
    ];
    grid.datasource.filter = conditions;
    grid.datasource.get().then(() => {
      grid.datasource.loading = false;
    });
  }
  
  public confirmDeletePlate({ component }: IEventParam<Button>) {
    const nameButton = component.name;
    const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
    
    if (grid.selectedRows.length === 0) {
      this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    } else {
      DialogService.show({
        name: nameButton,
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.excluir(grid, grid.selectedRows) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public excluir(grid: Grid, selectedRows: IDictionary) {
    const count: IDictionary = {
      PADRAO: {},
      LOCAL: {},
      SERVICO: {},
      TIPOCARD: {},
    };
        
    const rows = selectedRows.map((data: IDictionary) => {
      const {
        FILIAL, CDPRATO, CDFILAUXPRAT, CDSERVFILI, CDTIPOCARD, CDPRODUTO, CDFILCENTPME,
      } = data;

      let tipoReceita;
      if (CDSERVFILI) tipoReceita = 'SERVICO';
      else if (CDTIPOCARD) tipoReceita = 'TIPOCARD';
      else tipoReceita = (CDFILAUXPRAT === 'P') ? 'PADRAO' : 'LOCAL';

      const receita: IDictionary = count[tipoReceita];
      receita[CDPRATO] = Object.hasOwnProperty.call(receita, CDPRATO) ? receita[CDPRATO] += 1 : 1;
      return {
        FILIAL, CDPRATO, CDFILAUXPRAT, CDPRODUTO, CDSERVFILI, CDFILCENTPME, CDTIPOCARD,
      };
    });
    
    const {
      PADRAO, LOCAL,
    } = count;
    
    const buttonYes = Metadata.getInstance<Button>('yes');
    buttonYes.disabled = true;

    Http.post('/prato/deleteCheckedRows', { selectedRows: rows, PADRAO, LOCAL }).then(() => {
      this.utilities.showAlert('Alert', 'pme::excluidoComSucesso', 'success');
      grid.selectedRows = [];
      grid.reload();
      DialogService.hide();
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
      // if (error.includes('ORA-02292')) this.utilities.showAlert('alert', 'pme::ERROR_REGISTROFILHO', 'error');
    }).finally(() => {
      buttonYes.disabled = false;
    });
  }
  
  public alterarReceitas({ component }: IEventParam<Button>) {
    const form: Form = Metadata.getInstance('formFiltroUtilizacaoProd');
    const row = form.value;
    const pratoInicial = Metadata.getInstance<SelectTree>('PRATOINICIAL');
    const pratoFinal = Metadata.getInstance<SelectTree>('PRATOFINAL');
    if (row.TPRECEITA === null) row.TPRECEITA = row.TPRECEITA1;
    const validInitPlate = pratoInicial.validate();
    const validFinalPlate = pratoFinal.validate();
    if ((form.validate() || row.TPRECEITA) && (validInitPlate && validFinalPlate)) {
      const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
      const rowsGrid = grid.selectedRows;
      if (rowsGrid.length > 0) {
        if (row.TPRECEITA.includes('P')) {
          if (this.prato.operatorValid !== false) {
            if (component.name === 'salvarUtilizacaoProd') {
              this.alteracaoReceitas.openModalAteracaoReceitas('A'); // this.editar();
            } else if (component.name === 'addButtonUtil') {
              this.alteracaoReceitas.openModalAteracaoReceitas('I');
            }
          } else {
            this.utilities.showAlert('Alert', 'pme::modificacaoRecPadrao', 'warning');
            this.prato.openModalGroupOperador(component);
          }
        } else if (component.name === 'salvarUtilizacaoProd') {
          this.alteracaoReceitas.openModalAteracaoReceitas('A'); // this.editar();
        } else if (component.name === 'addButtonUtil') {
          this.alteracaoReceitas.openModalAteracaoReceitas('I');
        }
      } else this.utilities.showAlert('Alert', 'pme::aoMenosUmPratoSelecionado', 'error');
    }
  }
  
  public controllPratoIni({ component }: IEventParam<SelectTree>) {
    const pratoFinal = Metadata.getInstance<SelectTree>('PRATOFINAL');
    if (pratoFinal.value !== null) pratoFinal.setValue(null);
    this.setFilterUtilizacaoProdutos({ component });
  }
  
  public verifyPratIni({ component }: IEventParam<SelectTree>) {
    if (component.value !== null) {
      const pratoIni = Metadata.getInstance<Select>('PRATOINICIAL');
      if (this.utilities.returnPrat(pratoIni.value, component.value)) {
        this.utilities.showAlert('Alert', 'pme::pratoIniMaior', 'error');
        component.setValue(null);
      } else {
        this.setFilterUtilizacaoProdutos({ component });
      }
    }
  }
  
  public dataSourceGet({ component }: IEventParam<SelectTree>) {
    const unidade = Metadata.getInstance<Form>('formFiltroUtilizacaoProd').value.UNIDADE;
    const unidadeValue = (unidade.length !== 0) ? unidade.join('_') : null;
    const tprece = Metadata.getInstance<Form>('formFiltroUtilizacaoProd').value.TPRECEITA;
    // const unidade = Metadata.getInstance<Form>('formUnidade').value.UNIDADEFORM;
    if (component.datasource !== undefined) {
      component.datasource.filter = {
        CDPRATONIVANTE: component.datasource.filter.CDPRATONIVANTE,
        CDFILIAL: unidadeValue !== null && tprece !== null && tprece.includes('L') ? unidadeValue : null,
      };
      // component.datasource.get();
    }
  }
  
  public clearFields({ component }: IEventParam<Select>) {
    const form = Metadata.getInstance<Form>('formFiltroUtilizacaoProd');
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const servico = Metadata.getInstance<Select>('SERVICO');
    this.utilities.clearPeriodoParams();

    if (component.name === 'TIPOCARD_ACAO' || component.name === 'GRUPFILI') {
      servico.setValue('');
      unidade.setValue('');
    } else if (component.name === 'UNIDADE' || component.name === 'GRUPOSERVICO') {
      servico.setValue('');
    } else unidade.setValue('');
    
    if (form.value.PRATOINICIAL !== null && form.value.PRATOFINAL !== null) {
      this.setFilterUtilizacaoProdutos({ component });
    }
  }
}
